# first/last quantiles
fq(v) = quantile(v, 0.05)
lq(v) = quantile(v, 0.95)

function plot_m_vs_n(df;
		y_statistic = mean, 
		include_legend=true,
		include_ylabel=true,
		title=nothing)
	n = df.n[1]
	@assert all(df.n .== n)
	plotopts = Dict(:xscale => :log10, 
					:yscale => :log10,
					:legend => (include_legend ? :outerright : nothing),
					:margin => 0mm, :right_margin => -6mm)
	println(df)
	df_nys = df[df.method.=="Nyström",:]
	# println(df_nys[df_nys.m .== 1000,:])
	rdf_nys = combine(groupby(df_nys, :m), 
					  :rkhs_error => y_statistic => :ystat_err, 
					  :rkhs_error => std => :std_err, 
					  :rkhs_error => fq => :q05_err, 
					  :rkhs_error => lq => :q95_err)
	# println(rdf_nys)
	p = @df rdf_nys plot(:m, :ystat_err; ribbon=(:q05_err, :q95_err),
						 label=LaTeXString("Nyström"), 
						 plotopts...)
	# Baseline
	df_bl = df[df.method.=="Empirical",:rkhs_error]
	v(x) = repeat([x], 2)
	plot!([x for x in xlims()], v(y_statistic(df_bl)); ribbon=(v(fq(df_bl)), v(lq(df_bl))), 
		 label=LaTeXString("Empirical"), 
		  plotopts...)
	# vline!(sqrt(df[1,:n]); label=L"\sqrt{n}", style=:dash, color=:grey)
	vline!(sqrt(df[1,:n])*log10(sqrt(df[1,:n])); label=L"\sqrt{n}\log(\sqrt{n})", style=:dash, color=:grey)
	xlabel!("m")
	tit = (title == nothing) ? LaTeXString("\$n=$(n)\$") : title
	title!(tit)
	include_ylabel && ylabel!("Estimation error")
	return p
end

hline!(c; kwargs...) = plot!([xlims()[1], xlims()[2]], [c, c]; kwargs...)
vline!(c; kwargs...) = plot!([c, c], [ylims()[1], ylims()[2]]; kwargs...)
