using NyströmKME
using KernelFunctions
using DataFrames
using CSV
using FatDatasets
using Random

function main_synthetic_data(d, nb_its; seed=1234)
	Random.seed!(seed)
	p = random_GMM(8, d; intra_var=1.0, inter_intra_var_ratio=5.0)
	for n in 10 .^ [3, 4, 5]
		output="results_seed$(seed)_synthetic_n$(n).csv"
		df = DataFrame(exp=Int[], n=Int[], m=Union{Int, Missing}[], rkhs_error=Float64[], diff_rkhs_error=Float64[], method=String[]) 
		for it = 1:nb_its
			σ = estim_kernel_std(p)
			k = gaussian_kernel(σ)
			μ = DistributionKME(p, k, d)

			# Empirical estimator
			X = rand(p, n)
			μ_hat = EmpiricalKME(X, k)
			μhat_error = rkhs_dist(μ, μ_hat) 
			push!(df, Dict(:exp=>it, :n=>n, :m=>missing, :rkhs_error=>μhat_error, :diff_rkhs_error=>0.0,  :method=>"Empirical"))

			# Nyström estimator
			for m in round.(Int, 10 .^ range(log10(2), min(4, log10(n)), length=8))
				println("Iteration $it/$nb_its, n=$n, m=$m.")
				μ_Nys = Nyström_KME(X, k, m)
				error = rkhs_dist(μ, μ_Nys) 
				derror = abs(error - μhat_error)
				# store
				push!(df, Dict(:exp=>it, :n=>n, :m=>m, :rkhs_error=>error, :diff_rkhs_error=>derror,  :method=>"Nyström"))
			end

			println("Saving result, df has size $(size(df))")
			CSV.write(output, df)
		end
	end
end

const d = 10
const nb_its = 100
main_synthetic_data(d, nb_its)
