# @everywhere Pkg.activate(dirname(@__DIR__))
using Test
using NyströmKME
using KernelFunctions
using Distributions
using FatDatasets
N = NyströmKME
KF = KernelFunctions

d = 3
p = MvNormal(randn(d), randn(d))
n = 10^3
k = gaussian_kernel(3.0)
μ = DistributionKME(p, k, d)
X = rand(p, n)
μ_hat = EmpiricalKME(X, k)

@testset "Metrics" begin
	m = 10^1
	μ_Nys = Nyström_KME(X, k, m)

	@test rkhs_dist(μ_hat, μ_hat) ≈ 0.0
	@test rkhs_dist(μ, μ) ≈ 0.0
	@test rkhs_dist(μ_Nys, μ_Nys) ≈ 0.0
end

@testset "Nyström" begin
	m = n
	μ_Nys = Nyström_KME(X, k, m)
	# When using all points we should get the empirical estimator
	@test abs(rkhs_dist(μ_Nys, μ_hat)) < 1e-4 # This is already a huge error??
end

@testset "Speed-up" begin
	X = rand(p, n)
	Y = rand(p, n)
	wX = randn(n)
	wY = randn(n)
	f(X,Y) = kernelmatrix(k, X, Y)
	bsz = floor(Int, n/3)
	# @test (wX'*kernelmatrix(k, X, Y)*wY) ≈ N.batch_quadratic(f, X, Y, wX, wY; batchsize=bsz)
	@test N.kernel_quadratic(k, X, wX; batchsize=bsz) ≈ N.kernel_quadratic(k, X, X, wX, wX; batchsize=bsz)
	@test N.kernel_quadratic(k, X, wX; batchsize=bsz) ≈ wX'*kernelmatrix(k, X)*wX
	@test N.kernel_quadratic(k, X, Y, wX, wY; batchsize=bsz) ≈ wX'*kernelmatrix(k, X, Y)*wY
	b = max(size(X,2), size(Y,2))
	@test N.kernel_quadratic(k, X, wX; batchsize=b) ≈ wX'*kernelmatrix(k, X)*wX
	@test N.kernel_quadratic(k, X, Y, wX, wY; batchsize=b) ≈ wX'*kernelmatrix(k, X, Y)*wY
	@test N.kernel_quadratic(k, X, wX) ≈ wX'*kernelmatrix(k, X)*wX
	@test N.kernel_quadratic(k, X, Y, wX, wY) ≈ wX'*kernelmatrix(k, X, Y)*wY
end

@testset "H5" begin
	X = load_dataset("fasttext_en"; ncols=100)
	μ_hat = EmpiricalKME(X, k)
	@test rkhs_dist(μ_hat, μ_hat) ≈ 0.0
end
